/**
 * Tencent is pleased to support the open source community by making Tars available.
 *
 * Copyright (C) 2016THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the BSD 3-Clause License (the "License"); you may not use this file except 
 * in compliance with the License. You may obtain a copy of the License at
 *
 * https://opensource.org/licenses/BSD-3-Clause
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 * specific language governing permissions and limitations under the License.
 */

#include "servant/Global.h"
#include "servant/BaseF.h"
#include "servant/RemoteLogger.h"

namespace tars
{

//////////////////////////////////////////////////////////////////////////
void TarsException::throwException(int ret, const string& desc)
{
    switch (ret)
    {
    case tars::TARSSERVERSUCCESS:
        break;
    case TARSSERVERDECODEERR:
        throw TarsServerDecodeException("server decode exception: ret:" + TC_Common::tostr(ret) + " msg:"+ desc);
        break;
    case TARSSERVERENCODEERR:
        throw TarsServerEncodeException("server encode exception: ret:" + TC_Common::tostr(ret) + " msg:"+ desc);
        break;
    case TARSSERVERNOFUNCERR:
        throw TarsServerNoFuncException("server function mismatch exception: ret:" + TC_Common::tostr(ret) + " msg:"+ desc);
        break;
    case TARSSERVERNOSERVANTERR:
        throw TarsServerNoServantException("server servant mismatch exception: ret:" + TC_Common::tostr(ret) + " msg:"+ desc);
        break;
    case TARSSERVERQUEUETIMEOUT:
        throw TarsServerQueueTimeoutException("server queue timeout exception: ret:" + TC_Common::tostr(ret) + " msg:"+ desc);
        break;
    case TARSPROXYCONNECTERR:
        throw TarsServerConnectionException("server connection lost: ret:" + TC_Common::tostr(ret) + " msg:"+ desc);
        break;
    case TARSINVOKETIMEOUT:
		throw TarsServerInvokeTimeoutException(desc);
	default:
    throw TarsServerUnknownException("server unknown exception: ret:" + TC_Common::tostr(ret) + " msg:"+ desc);
    }
}
////////////////////////////////////////////////////////////////////////////
}
